// this is a test bed for a socket wrappers
// and a demo how to use them
// in fact, this is the server and the client
// combined - use:
// test c   - for starting it as a client and
// test s   - for starting it as a server

#include "Sockets.h"
#include <iostream>
#include <string>
#include <complex>

using namespace std;

// address of the server
// for testing, both server and client are on the same machine
const int IPportnumber = 12345;
const char *IPserveraddress = "127.0.0.1";

void do_server()
{
	string message("Hello Socket!");
	complex<double> cplx(1.0, 2.0);
	int integer(1234);

	try
	{
		TCPSocketWrapper sockserver;

		// listen on some port
		sockserver.listen(IPportnumber);

		cout << "server is ready" << endl;

		// accept connection from client
		TCPSocketWrapper sock(sockserver.accept());

		cout << "accepted connection from: "
			<< sock.address() << endl;

		// make the stream around the socket wrapper
		TCPStream stream(sock);

		bool oncemore = true;
		int command;
		while (oncemore)
		{
			// read the command
			stream >> command;
			switch (command)
			{
			case 1:
				cout << "command 1" << endl;
				stream << message << endl;
				break;
			case 2:
				cout << "command 2" << endl;
				stream << integer << endl;
				break;
			case 3:
				cout << "command 3" << endl;
				stream << cplx << endl;
				break;
			default:
				cout << "END command" << endl;
				oncemore = false;
				break;
			}
		}
	}
	catch (const SocketRunTimeException &e)
	{
		cout << "socket exception: "
			<< e.what() << endl;
	}
}

void do_client()
{
	string message;
	complex<double> cplx;
	int integer;

	try
	{
		TCPClientStream stream(IPserveraddress, IPportnumber);

		bool oncemore = true;
		int command;
		while (oncemore)
		{
			cout << "1. request for a string" << endl;
			cout << "2. request for a number" << endl;
			cout << "3. request for a complex number" << endl;
			cout << "other - end" << endl;

			cin >> command;
			stream << command << endl;

			switch (command)
			{
			case 1:
				do
				{
					getline(stream, message);
				} while (message.empty());
				cout << "received: " << message << endl;
				break;
			case 2:
				stream >> integer;
				cout << "received: " << integer << endl;
				break;
			case 3:
				stream >> cplx;
				cout << "received: " << cplx << endl;
				break;
			default:
				oncemore = false;
				break;
			}
		}
	}
	catch (const SocketRunTimeException &e)
	{
		cout << "socket exception: " << e.what() << endl;
	}
}

int main(int argc, char *argv[])
{
	if (argc < 2 || (argv[1][1] != 'c' && argv[1][1] != 's'))
	{
		cout << "use:\ntest -c   for client process"
			<< "\ntest -s   for server process" << endl;
		return 0;
	}

	if (socketsInit() == false)
	{
		cout << "cannot initialize sockets" << endl;
		return 0;
	}

	if (argv[1][1] == 's')
		do_server();
	else
		do_client();

	socketsEnd();

	return 0;
}
